/**
 * Module dependencies.
 */

var Lexer = require('./lib/lexer');

/**
 * Module exports.
 */

exports = module.exports = markdown;

/**
 * Markdown parser.
 * @param text
 */

function markdown(text) {

  // Create a new Lexer:
  var lexem = new Lexer()
    .def('whitespace', /^[\t \n]$/)
    .def('dstar', /^[\*]{2}$/)
    //.def('sstar', /^[\*]{1}$/)
    .def('string', /^[A-Za-z0-9\.\-\^\%\$\@\!\~\`\+\=\-\_]+$/)
    .def('dunderscore', /^[\_]{2}$/)
    //.def('sunderscore', /^[\_]{1}$/)
    .def('left_square_bracket', /^\[$/)
    .def('right_square_bracket', /^\]$/)
    .def('left_parenthesis', /^\($/)
    .def('right_parenthesis', /^\)$/)
    .def('h1', /^[#]$/)
    .def('h2', /^[#]{2}$/)
    .def('h3', /^[#]{3}$/)
    .def('h4', /^[#]{4}$/)
    .def('h5', /^[#]{4}$/)
    .def('h6', /^[#]{4}$/)
    .string(text)
    .start();


  function parseDoubleStars(tree) {
    var token = lexem.token;

    var double = {
      type: 'strong',
      value: []
    };

    lexem.next();
    while(lexem.token.type !== 'dstar' || lexem.token.value === 'EOF') {
      double.value = parse(double.value);
      lexem.next();
    }

    if (lexem.token.type === 'dstar') {
      // Yeah!!
      tree.push(double);
    }

    lexem.next();
    return parse(tree);
  }

  function parseString(tree) {
    var val = lexem.token.value;
    tree.push({
      type: 'string',
      value: val
    });
    return tree;
  }

  function parseDoubleUnderscores(tree) {
    var token = lexem.token;

    var double = {
      type: 'strong',
      value: []
    };

    lexem.next();

    while(lexem.token.type !== 'dunderscore' || lexem.token.value !== 'EOF') {
      double.value = parse(double.value);
      lexem.next();
    }

    if (lexem.token.type === 'dunderscore') {
      // Yeah!!
      tree.push(double);
    }

    lexem.next();
    return parse(tree);
  }

  function parseAnchor() {

  }

  function parseHeaders(tree) {
    var token = lexem.token;

    var c = {
      type: 'header',
      value: [],
      n: token.type.replace('h', '')
    };

    lexem.next();
    while (lexem.token.type === 'string' || lexem.token.type === 'whitespace') {
      c.value = parse(c.value);
      lexem.next();
    }

    tree.push(c);

    return tree;
  }

  function parseWhitespace(tree) {
    tree.push({
      type: 'whitespace',
      value: lexem.token.value
    });

    lexem.next();
    return parse(tree);
  }

  function parse(tree) {

    var type = lexem.token.type;
    if (type === 'EOF' || type === 'eof') return tree;
    console.log(type);
    switch(type) {
      case 'whitespace':
        return parseWhitespace(tree);
        break;
      case 'dstar':
        return parseDoubleStars(tree);
        break;
      case 'dunderscore':
        return parseDoubleUnderscores(tree);
        break;
      case 'left_square_bracket':
        return parseAnchor(tree);
        break;
      case 'h1':
      case 'h2':
      case 'h3':
      case 'h4':
      case 'h5':
      case 'h6':
        return parseHeaders(tree);
        break;
      case 'string':
        return parseString(tree);
        break;
     default:
      return tree;
      break;
    }
  }

  lexem.next();
  var tree = [];
  tree = parse(tree);
  console.log(tree);
}
